@@ -330,6 +330,8 @@ static int parse_DHT(struct jdec_private *priv, const unsigned char *stream)
 
         Tc = index & 0xf0; // it is not important to <<4
         Th = index & 0x0f;
+        if (Th >= HUFFMAN_TABLES)
+            error("No more than %d Huffman tables is supported (got %d)\n", HUFFMAN_TABLES, Th);
         if (Tc) {
             memcpy(priv->HTAC[Th].bits, stream, 16);
         } else {
@@ -344,8 +346,6 @@ static int parse_DHT(struct jdec_private *priv, const unsigned char *stream)
 #if SANITY_CHECK
         if (count >= HUFFMAN_BITS_SIZE)
             error("No more than %d bytes is allowed to describe a huffman table", HUFFMAN_BITS_SIZE);
-        if (Th >= HUFFMAN_TABLES)
-            error("No more than %d Huffman tables is supported (got %d)\n", HUFFMAN_TABLES, Th);
         trace("Huffman table %s[%d] length=%d\n", Tc ? "AC" : "DC", Th, count);
 #endif
 
@@ -486,6 +486,13 @@ static int parse_JFIF(struct jdec_private *priv, const unsigned char *stream)
         printf("ERROR:Sampling other than 1x1 for Cr and Cb is not supported");
 #endif
     findEOI(priv, stream);
+
+    if (priv->stream_scan > priv->stream_end)
+    {
+        error("stream_scan cannot exceed stream_end\n");
+        return -1;
+    }
+
     return next_image_found;
 }
 
@@ -816,6 +823,8 @@ int tinyjpeg_decode(struct jdec_private *priv)
         vaDestroyConfig(va_dpy, config_id);
         vaDestroyContext(va_dpy, context_id);
 
+        if (scan_num >= JPEG_SCAN_MAX)
+            break;
         parse_JFIF(priv, priv->stream);
         if (priv->width[scan_num] == 0 && priv->height[scan_num] == 0)
             break;
